package backend;

import intermediate.SymtabEntry;

import java.util.ArrayList;

/**
 * The runtime stack and its display.
 */
public class RuntimeStack 
{
    private static ArrayList<ActivationRecord> stack   = new ArrayList<>();
    private static ArrayList<ActivationRecord> display = new ArrayList<>();
    private static int tos = -1;
    
    // Add a dummy zeroth element to the display
    // in order to index starting with nesting level 1.
    static {
        display.add(null);
    }
    
    /**
     * Push an activation record onto the stack.
     * @param record the activation record.
     */
    public static void push(ActivationRecord record)
    {
        ++tos;
        stack.add(record);
        
        int level = record.getLevel();
        
        if (display.size() <= level) {
            display.add(record);
        }
        else {
            // Set the activation record's back link.
            record.setLink(display.get(level));
            display.set(level, record);
        }
    }
    
    /**
     * Pop off and return the activation record at the top of the stack.
     * @return the popped-off activation record.
     */
    public static ActivationRecord pop()
    {
        ActivationRecord top = stack.get(tos);
        int level = top.getLevel();
        
        // Reset the display using the activation record's back link.
        display.set(level, top.getLink());
        
        return stack.remove(tos--);
    }
    
    /**
     * Retrieve a variable's value from the activation record.
     * @param entry the variable's symbol table entry.
     * @return the value.
     */
    public static Object getValue(SymtabEntry entry)
    {
        int level = entry.getSymtab().getLevel();
        String name = entry.getName();
        
        // Get the value from the appropriate activation record.
        ActivationRecord record = display.get(level);
        return record.getValue(name);
    }
    
    /**
     * Set a variable's value in the activation record.
     * @param entry the variable's symbol table entry.
     * @param value the value to set.
     * @return the value that was set.
     */
    public static Object setValue(SymtabEntry entry, Object value)
    {
        int level = entry.getSymtab().getLevel();
        String name = entry.getName();
        
        // Set the value in the appropriate activation record.
        ActivationRecord record = display.get(level);
        record.setValue(name, value);
        
        return value;
    }
}
